@@ -3,8 +3,8 @@
 
 title = 'flaskcode'
 
-numversion = (0, 0, 3)
-version = '.'.join([str(num) for num in numversion])
+version_info = (0, 0, 8)
+version = '.'.join(map(str, version_info))
 
 license = 'MIT'
 copyright = 'Copyright (c) 2019 Sujeet Kumar <sujeetkv90@gmail.com>'
@@ -13,6 +13,6 @@ author = 'Sujeet Kumar'
 email = 'sujeetkv90@gmail.com'
 uri = 'https://github.com/sujeetkv/flaskcode'
 
-description = 'Web based code editor for flask'
+description = 'Web based code editor on python flask framework'
 
 status = 'Development'
