@@ -3,8 +3,6 @@
 
 var flaskcode = window.flaskcode || {};
 
-flaskcode.$pagePreloader = null;
-
 require.config({
     baseUrl: flaskcode.config.get('pluginsBaseUrl'),
     paths: {'vs': 'monaco-editor/min/vs'}
@@ -26,6 +24,7 @@ flaskcode.fallbackLang = null;
 flaskcode.$editorContainer = null;
 flaskcode.$editorBody = null;
 flaskcode.$editorLoader = null;
+flaskcode.$pagePreloader = null;
 
 flaskcode.editorStates = {
     INIT: 'init',
@@ -34,12 +33,165 @@ flaskcode.editorStates = {
     BUSY: 'busy',
 };
 
+flaskcode.defaultEditorTheme = 'vs-dark';
+flaskcode.availableEditorThemes = ['vs', 'vs-dark', 'hc-black'];
+
 flaskcode.APP_BUSY = false;
 
-flaskcode.allowedLangIds = ['plaintext', 'html', 'css', 'less', 'scss', 'javascript', 'typescript', 'coffeescript', 'json', 'markdown'];
+flaskcode.allowedLangIds = [];
 
 flaskcode.onStateChange = $.noop;
 
+$(function () {
+    flaskcode.$pagePreloader = $('div#page-preloader');
+    flaskcode.$editorContainer = $('div#editor-container');
+    flaskcode.$editorBody = flaskcode.$editorContainer.find('.editor-body');
+    flaskcode.$editorLoader = flaskcode.$editorContainer.find('.editor-preloader');
+    flaskcode.editorElement = flaskcode.$editorBody.get(0);
+
+    $('ul#dir-tree').treed();
+    flaskcode.setEditorState(flaskcode.editorStates.INIT);
+
+    require(['vs/editor/editor.main'], function() {
+        flaskcode.languages = monaco.languages.getLanguages();
+        flaskcode.allowedLangIds = flaskcode.languages.map(function (lang) { return lang.id; });
+        flaskcode.defaultLang = flaskcode.getLanguageByExtension(flaskcode.defaultExt);
+        flaskcode.fallbackLang = flaskcode.getLanguageById(flaskcode.defaultLangId);
+
+        $('ul#dir-tree').on('click', '.file-item', function () {
+            return flaskcode.openResource($(this));
+        });
+
+        $('.header-actions').on('click', function () {
+            if (flaskcode.editorWidget.editor) {
+                flaskcode.editorWidget.editor.trigger('mouse', $(this).data('actionId'));
+            }
+        });
+
+        $('#toggle-minimap').on('click', function () {
+            flaskcode.minimapEnabled(!flaskcode.minimapEnabled());
+        });
+
+        $.contextMenu({
+            selector: 'ul#dir-tree .resource-items',
+            autoHide: false,
+            build: function ($trigger, e) {
+                var items = {
+                    'title': {name: $trigger.data('pathName'), icon: 'fa-tag', disabled: true},
+                    'sep': '---------'
+                };
+                if ($trigger.hasClass('dir-item')) {
+                    items['title']['icon'] = 'fa-folder';
+                    if ($trigger.hasClass('expanded')) {
+                        items['toggle_collapse'] = {name: 'Collapse', icon: 'fa-compress'};
+                    } else {
+                        items['toggle_collapse'] = {name: 'Expand', icon: 'fa-expand'};
+                    }
+                    items['create_new_file'] = {name: 'Create New File', icon: 'fa-file-code-o'};
+                } else {
+                    items['title']['icon'] = 'fa-file';
+                    items['open'] = {name: 'Open', icon: 'fa-external-link', disabled: $trigger.hasClass('selected')};
+                }
+                return {
+                    items: items,
+                    callback: function (key, options) {
+                        switch (key) {
+                            case 'toggle_collapse':
+                                options.$trigger.click();
+                                break;
+    
+                            case 'open':
+                                flaskcode.openResource(options.$trigger);
+                                break;
+    
+                            case 'create_new_file':
+                                flaskcode.openNewFileModal(options.$trigger);
+                                break;
+                        }
+                    },
+                };
+            },
+            events: {
+                show: function (options) {},
+                hide: function (options) {},
+            },
+        });
+    
+        $('#fileNameModal').on('shown.bs.modal', function () {
+            $(this).find('#new_filename').focus();
+        });
+
+        $('form#fileNameForm').on('submit', function (evt) {
+            evt.preventDefault();
+            var $form = $(this);
+            var $button = $form.find('[type="submit"]');
+            var base_url = $form.find('#base_url').val();
+            var base_path_name = $form.find('#base_path_name').val();
+            var new_filename = $form.find('#new_filename').val();
+            if (!(new_filename && flaskcode.validResource(new_filename))) {
+                $form.find('.form-msg').empty().append($('<span class="text-danger">Please enter valid file name.</span>').autoremove(10));
+            } else {
+                var resource_url = base_url + '/' + new_filename + '.txt';
+                $.ajax({
+                    type: 'HEAD',
+                    url: resource_url,
+                    dataType: 'text',
+                    cache: false,
+                    beforeSend: function (xhr, settings) {
+                        $button.button('loading');
+                    },
+                    complete: function (xhr, status) {
+                        $button.button('reset');
+                    },
+                }).done(function (data, status, xhr) {
+                    $form.find('.form-msg').empty().append($('<span class="text-danger">This file already exists.</span>').autoremove(10));
+                }).fail(function (xhr, status, err) {
+                    if (xhr.status == 404) {
+                        flaskcode.loadEditor({url: resource_url, filePath: base_path_name + '/' + new_filename}, false, true);
+                        $('#fileNameModal').modal('hide');
+                    } else {
+                        $form.find('.form-msg').empty().append($('<span class="text-danger">Internal Error: '+err+'</span>').autoremove(10));
+                    }
+                });
+            }
+            return false;
+        });
+
+        $('#editor-header #resource-close').on('click', function () {
+            if (flaskcode.editorWidget.editorState == flaskcode.editorStates.MODIFIED && !confirm('Close without saving?')) {
+                return false;
+            }
+            flaskcode.saveResourceState();
+            flaskcode.clearEditor();
+            flaskcode.resetSelectedResource();
+            $('#editor-header #resource-mod').hide();
+            $('#editor-header #resource-name').text('').attr('title', '');
+            $(this).hide();
+        });
+
+        $(window).on('resize', function () {
+            if (flaskcode.editorWidget.editor) {
+                flaskcode.editorWidget.editor.layout();
+            }
+        });
+
+        $(window).on('beforeunload', function (evt) {
+            if (flaskcode.APP_BUSY || flaskcode.editorWidget.editorState == flaskcode.editorStates.BUSY) {
+                evt.preventDefault();
+                evt.returnValue = 'Editor is working...';
+                return evt.returnValue;
+            }
+        });
+
+        flaskcode.$pagePreloader.fadeOut();
+    });
+});
+
+flaskcode.editorTheme = function () {
+    var theme = flaskcode.config.get('editorTheme', flaskcode.defaultEditorTheme);
+    return flaskcode.availableEditorThemes.indexOf(theme) > -1 ? theme : flaskcode.defaultEditorTheme;
+};
+
 flaskcode.setEditorState = function (state) {
     var prevState = flaskcode.editorWidget.editorState;
     flaskcode.editorWidget.editorState = state;
@@ -55,7 +207,7 @@ flaskcode.getExt = function (filename) {
 
 flaskcode.getResourceExt = function (resource_url) {
     var m = resource_url.match(/\.(\w*)\.txt$/i);
-    return (m && m.length > 1) ? m[1].toLowerCase() : flaskcode.defaultExt;
+    return (m && m.length > 1) ? m[1].toLowerCase() : null;
 };
 
 flaskcode.getLanguageById = function (langId) {
@@ -91,7 +243,7 @@ flaskcode.minimapEnabled = function (minimapFlag) {
 flaskcode.notifyEditor = function (message, category) {
     var msgType = category == 'error' ? 'danger' : 'success';
     var $msg = $('<div class="alert alert-dismissible alert-'+msgType+'" role="alert">'+
-        '<button type="button" class="close" data-dismiss="alert" aria-label="Close"><span aria-hidden="true">&times;</span></button>'+
+        '<button type="button" class="close" data-dismiss="alert" aria-label="Close"><i class="fa fa-times" aria-hidden="true"></i></button>'+
         '<strong>'+message+'</strong>'+
     '</div>');
     $msg.autoremove(msgType == 'success' ? 5 : 10);
@@ -135,7 +287,7 @@ flaskcode.highlightSelectedResource = function (filePath, parentPath) {
             }
         }
     }
-    $('#editor-header #resource-name').text(flaskcode.strTruncateLeft(filePath)).attr('title', filePath);
+    $('#editor-header #resource-name').text(flaskcode.strTruncateLeft(filePath, 40)).attr('title', filePath);
 };
 
 flaskcode.notifyCursorPosition = function (position) {
@@ -148,6 +300,14 @@ flaskcode.notifyCursorPosition = function (position) {
     }
 };
 
+flaskcode.notifyLanguage = function (lang) {
+    if (lang && lang.aliases.length) {
+        $('span#editor_lang').text(lang.aliases[0]);
+    } else {
+        $('span#editor_lang').text('');
+    }
+};
+
 flaskcode.onEditorStateChange = function (state) {
     if (state == flaskcode.editorStates.LOADED) {
         $('#resource-close').show();
@@ -306,13 +466,13 @@ flaskcode.restoreResourceState = function () {
 };
 
 flaskcode.setEditor = function (data, resource, isNewResource) {
-    var resourceLang = flaskcode.getLanguageByExtension(resource.extension || flaskcode.getResourceExt(resource.url)) || flaskcode.getLanguageByMimetype(resource.mimetype);
+    var resourceLang = flaskcode.getLanguageByExtension(flaskcode.getResourceExt(resource.url) || resource.extension || flaskcode.defaultExt) || flaskcode.getLanguageByMimetype(resource.mimetype);
     var lang = resourceLang || flaskcode.defaultLang || flaskcode.fallbackLang;
 
     if (!flaskcode.editorWidget.editor) {
         flaskcode.resetEditorBody();
         flaskcode.editorWidget.editor = monaco.editor.create(flaskcode.editorElement, {
-            theme: 'vs-dark',
+            theme: flaskcode.editorTheme(),
             minimap: {enabled: flaskcode.minimapEnabled()},
             fontSize: 13,
             model: null,
@@ -332,6 +492,7 @@ flaskcode.setEditor = function (data, resource, isNewResource) {
     flaskcode.setEditorState(flaskcode.editorStates.LOADED);
     flaskcode.editorWidget.editor.focus();
     flaskcode.notifyCursorPosition(flaskcode.editorWidget.editor.getPosition());
+    flaskcode.notifyLanguage(lang);
 
     if (oldModel) {
         oldModel.dispose();
@@ -350,6 +511,7 @@ flaskcode.clearEditor = function (alt_content) {
     }
     flaskcode.resetEditorBody().append(alt_content || '');
     flaskcode.notifyCursorPosition(null);
+    flaskcode.notifyLanguage(null);
 };
 
 flaskcode.loadEditor = function (resource, forceReload, isNewResource) {
@@ -418,147 +580,3 @@ flaskcode.openNewFileModal = function ($resourceElement) {
     $modal.find('#new_filename').val('');
     $modal.modal({show: true, backdrop: 'static'});
 };
-
-$(function () {
-    flaskcode.$pagePreloader = $('div#page-preloader');
-    flaskcode.$editorContainer = $('div#editor-container');
-    flaskcode.$editorBody = flaskcode.$editorContainer.find('.editor-body');
-    flaskcode.$editorLoader = flaskcode.$editorContainer.find('.editor-preloader');
-    flaskcode.editorElement = flaskcode.$editorBody.get(0);
-
-    $('ul#dir-tree').treed();
-    flaskcode.setEditorState(flaskcode.editorStates.INIT);
-
-    require(['vs/editor/editor.main'], function() {
-        flaskcode.languages = monaco.languages.getLanguages();
-        flaskcode.defaultLang = flaskcode.getLanguageByExtension(flaskcode.defaultExt);
-        flaskcode.fallbackLang = flaskcode.getLanguageById(flaskcode.defaultLangId);
-
-        $('ul#dir-tree').on('click', '.file-item', function () {
-            return flaskcode.openResource($(this));
-        });
-
-        $('.header-actions').on('click', function () {
-            if (flaskcode.editorWidget.editor) {
-                flaskcode.editorWidget.editor.trigger('mouse', $(this).data('actionId'));
-            }
-        });
-
-        $('#toggle-minimap').on('click', function () {
-            flaskcode.minimapEnabled(!flaskcode.minimapEnabled());
-        });
-
-        $.contextMenu({
-            selector: 'ul#dir-tree .resource-items',
-            autoHide: false,
-            build: function ($trigger, e) {
-                var items = {
-                    'title': {name: $trigger.data('pathName'), icon: 'fa-tag', disabled: true},
-                    'sep': '---------'
-                };
-                if ($trigger.hasClass('dir-item')) {
-                    items['title']['icon'] = 'fa-folder';
-                    if ($trigger.hasClass('expanded')) {
-                        items['toggle_collapse'] = {name: 'Collapse', icon: 'fa-compress'};
-                    } else {
-                        items['toggle_collapse'] = {name: 'Expand', icon: 'fa-expand'};
-                    }
-                    items['create_new_file'] = {name: 'Create New File', icon: 'fa-file-code-o'};
-                } else {
-                    items['title']['icon'] = 'fa-file';
-                    items['open'] = {name: 'Open', icon: 'fa-external-link', disabled: $trigger.hasClass('selected')};
-                }
-                return {
-                    items: items,
-                    callback: function (key, options) {
-                        switch (key) {
-                            case 'toggle_collapse':
-                                options.$trigger.click();
-                                break;
-    
-                            case 'open':
-                                flaskcode.openResource(options.$trigger);
-                                break;
-    
-                            case 'create_new_file':
-                                flaskcode.openNewFileModal(options.$trigger);
-                                break;
-                        }
-                    },
-                };
-            },
-            events: {
-                show: function (options) {},
-                hide: function (options) {},
-            },
-        });
-    
-        $('#fileNameModal').on('shown.bs.modal', function () {
-            $(this).find('#new_filename').focus();
-        });
-
-        $('form#fileNameForm').on('submit', function (evt) {
-            evt.preventDefault();
-            var $form = $(this);
-            var $button = $form.find('[type="submit"]');
-            var base_url = $form.find('#base_url').val();
-            var base_path_name = $form.find('#base_path_name').val();
-            var new_filename = $form.find('#new_filename').val();
-            if (!(new_filename && flaskcode.validResource(new_filename))) {
-                $form.find('.form-msg').empty().append($('<span class="text-danger">Please enter valid file name.</span>').autoremove(10));
-            } else {
-                var resource_url = base_url + '/' + new_filename + '.txt';
-                $.ajax({
-                    type: 'HEAD',
-                    url: resource_url,
-                    dataType: 'text',
-                    cache: false,
-                    beforeSend: function (xhr, settings) {
-                        $button.button('loading');
-                    },
-                    complete: function (xhr, status) {
-                        $button.button('reset');
-                    },
-                }).done(function (data, status, xhr) {
-                    $form.find('.form-msg').empty().append($('<span class="text-danger">This file already exists.</span>').autoremove(10));
-                }).fail(function (xhr, status, err) {
-                    if (xhr.status == 404) {
-                        flaskcode.loadEditor({url: resource_url, filePath: base_path_name + '/' + new_filename}, false, true);
-                        $('#fileNameModal').modal('hide');
-                    } else {
-                        $form.find('.form-msg').empty().append($('<span class="text-danger">Internal Error: '+err+'</span>').autoremove(10));
-                    }
-                });
-            }
-            return false;
-        });
-
-        $('#editor-header #resource-close').on('click', function () {
-            if (flaskcode.editorWidget.editorState == flaskcode.editorStates.MODIFIED && !confirm('Close without saving?')) {
-                return false;
-            }
-            flaskcode.saveResourceState();
-            flaskcode.clearEditor();
-            flaskcode.resetSelectedResource();
-            $('#editor-header #resource-mod').hide();
-            $('#editor-header #resource-name').text('').attr('title', '');
-            $(this).hide();
-        });
-
-        $(window).on('resize', function () {
-            if (flaskcode.editorWidget.editor) {
-                flaskcode.editorWidget.editor.layout();
-            }
-        });
-
-        $(window).on('beforeunload', function (evt) {
-            if (flaskcode.APP_BUSY || flaskcode.editorWidget.editorState == flaskcode.editorStates.BUSY) {
-                evt.preventDefault();
-                evt.returnValue = 'App is working...';
-                return evt.returnValue;
-            }
-        });
-
-        flaskcode.$pagePreloader.fadeOut();
-    });
-});
